@@ -18,7 +18,7 @@
 
 #include "pg_stat_monitor.h"
 
-GucVariable conf[13];
+GucVariable conf[MAX_SETTINGS];
 static void DefineIntGUC(GucVariable *conf);
 static void DefineBoolGUC(GucVariable *conf);
 
@@ -68,7 +68,7 @@ init_guc(void)
 	conf[i] = (GucVariable) {
 		.guc_name = "pg_stat_monitor.pgsm_track_utility",
 		.guc_desc = "Selects whether utility commands are tracked.",
-		.guc_default = 0,
+		.guc_default = 1,
 		.guc_min = 0,
 		.guc_max = 0,
 		.guc_restart = false,
@@ -161,6 +161,31 @@ init_guc(void)
 	};
 	DefineIntGUC(&conf[i++]);
 
+	conf[i] = (GucVariable) {
+		.guc_name = "pg_stat_monitor.pgsm_overflow_target",
+		.guc_desc = "Sets the overflow target for pg_stat_monitor",
+		.guc_default = 1,
+		.guc_min = 0,
+		.guc_max = 1,
+		.guc_restart = true,
+		.guc_unit = 0,
+		.guc_value = &PGSM_OVERFLOW_TARGET
+	};
+	DefineIntGUC(&conf[i++]);
+
+	conf[i] = (GucVariable) {
+		.guc_name = "pg_stat_monitor.pgsm_enable_query_plan",
+		.guc_desc = "Enable/Disable query plan monitoring",
+		.guc_default = 0,
+		.guc_min = 0,
+		.guc_max = 0,
+		.guc_restart = false,
+		.guc_unit = 0,
+		.guc_value = &PGSM_QUERY_PLAN
+	};
+	DefineBoolGUC(&conf[i++]);
+
+
 #if PG_VERSION_NUM >= 130000
 	conf[i] = (GucVariable) {
 		.guc_name = "pg_stat_monitor.pgsm_track_planning",
